@@ -1,14 +1,14 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #ifndef OPENVDB_MATH_MAT3_H_HAS_BEEN_INCLUDED
 #define OPENVDB_MATH_MAT3_H_HAS_BEEN_INCLUDED
 
 #include <openvdb/Exceptions.h>
+#include <openvdb/util/Assert.h>
 #include "Vec3.h"
 #include "Mat.h"
 #include <algorithm> // for std::copy()
-#include <cassert>
 #include <cmath>
 #include <iomanip>
 
@@ -34,23 +34,9 @@ public:
     using MyBase = Mat<3, T>;
 
     /// Trivial constructor, the matrix is NOT initialized
-#if OPENVDB_ABI_VERSION_NUMBER >= 8
     /// @note destructor, copy constructor, assignment operator and
     ///   move constructor are left to be defined by the compiler (default)
     Mat3() = default;
-#else
-    Mat3() {}
-
-    /// Copy constructor
-    Mat3(const Mat<3, T> &m)
-    {
-        for (int i=0; i<3; ++i) {
-            for (int j=0; j<3; ++j) {
-                MyBase::mm[i*3 + j] = m[i][j];
-            }
-        }
-    }
-#endif
 
     /// Constructor given the quaternion rotation, e.g.    Mat3f m(q);
     /// The quaternion is normalized and used to construct the matrix
@@ -154,7 +140,7 @@ public:
     /// Set ith row to vector v
     void setRow(int i, const Vec3<T> &v)
     {
-        // assert(i>=0 && i<3);
+        OPENVDB_ASSERT(i>=0 && i<3);
         int i3 = i * 3;
 
         MyBase::mm[i3+0] = v[0];
@@ -165,14 +151,14 @@ public:
     /// Get ith row, e.g.    Vec3d v = m.row(1);
     Vec3<T> row(int i) const
     {
-        // assert(i>=0 && i<3);
+        OPENVDB_ASSERT(i>=0 && i<3);
         return Vec3<T>((*this)(i,0), (*this)(i,1), (*this)(i,2));
     } // rowColumnTest
 
     /// Set jth column to vector v
     void setCol(int j, const Vec3<T>& v)
     {
-        // assert(j>=0 && j<3);
+        OPENVDB_ASSERT(j>=0 && j<3);
         MyBase::mm[0+j] = v[0];
         MyBase::mm[3+j] = v[1];
         MyBase::mm[6+j] = v[2];
@@ -181,7 +167,7 @@ public:
     /// Get jth column, e.g.    Vec3d v = m.col(0);
     Vec3<T> col(int j) const
     {
-        // assert(j>=0 && j<3);
+        OPENVDB_ASSERT(j>=0 && j<3);
         return Vec3<T>((*this)(0,j), (*this)(1,j), (*this)(2,j));
     } // rowColumnTest
 
@@ -190,8 +176,8 @@ public:
     /// e.g.    m(0,0) = 1;
     T& operator()(int i, int j)
     {
-        // assert(i>=0 && i<3);
-        // assert(j>=0 && j<3);
+        OPENVDB_ASSERT(i>=0 && i<3);
+        OPENVDB_ASSERT(j>=0 && j<3);
         return MyBase::mm[3*i+j];
     } // trivial
 
@@ -200,8 +186,8 @@ public:
     /// e.g.    float f = m(1,0);
     T operator()(int i, int j) const
     {
-        // assert(i>=0 && i<3);
-        // assert(j>=0 && j<3);
+        OPENVDB_ASSERT(i>=0 && i<3);
+        OPENVDB_ASSERT(j>=0 && j<3);
         return MyBase::mm[3*i+j];
     } // trivial
 
@@ -848,10 +834,8 @@ using Mat3s = Mat3<float>;
 using Mat3d = Mat3<double>;
 using Mat3f = Mat3d;
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 8
 OPENVDB_IS_POD(Mat3s)
 OPENVDB_IS_POD(Mat3d)
-#endif
 
 } // namespace math
 
