@@ -1,10 +1,11 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #ifndef OPENVDB_MATH_VEC3_HAS_BEEN_INCLUDED
 #define OPENVDB_MATH_VEC3_HAS_BEEN_INCLUDED
 
 #include <openvdb/Exceptions.h>
+#include <openvdb/util/Assert.h>
 #include "Math.h"
 #include "Tuple.h"
 #include <algorithm>
@@ -27,13 +28,9 @@ public:
     using ValueType = T;
 
     /// Trivial constructor, the vector is NOT initialized
-#if OPENVDB_ABI_VERSION_NUMBER >= 8
     /// @note destructor, copy constructor, assignment operator and
     ///   move constructor are left to be defined by the compiler (default)
     Vec3() = default;
-#else
-    Vec3() {}
-#endif
 
     /// @brief Construct a vector all of whose components have the given value.
     explicit Vec3(T val) { this->mm[0] = this->mm[1] = this->mm[2] = val; }
@@ -232,8 +229,8 @@ public:
     /// this = v1 cross v2, v1 and v2 must be distinct objects than "this"
     const Vec3<T>& cross(const Vec3<T> &v1, const Vec3<T> &v2)
     {
-        // assert(this!=&v1);
-        // assert(this!=&v2);
+        OPENVDB_ASSERT(this!=&v1);
+        OPENVDB_ASSERT(this!=&v2);
         this->mm[0] = v1.mm[1]*v2.mm[2] - v1.mm[2]*v2.mm[1];
         this->mm[1] = v1.mm[2]*v2.mm[0] - v1.mm[0]*v2.mm[2];
         this->mm[2] = v1.mm[0]*v2.mm[1] - v1.mm[1]*v2.mm[0];
@@ -667,12 +664,10 @@ using Vec3ui = Vec3<uint32_t>;
 using Vec3s = Vec3<float>;
 using Vec3d = Vec3<double>;
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 8
 OPENVDB_IS_POD(Vec3i)
 OPENVDB_IS_POD(Vec3ui)
 OPENVDB_IS_POD(Vec3s)
 OPENVDB_IS_POD(Vec3d)
-#endif
 
 } // namespace math
 } // namespace OPENVDB_VERSION_NAME
