@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 /// @file points/AttributeArray.h
 ///
@@ -14,6 +14,7 @@
 #include <openvdb/math/QuantizedUnitVec.h>
 #include <openvdb/util/Name.h>
 #include <openvdb/util/logging.h>
+#include <openvdb/util/Assert.h>
 #include <openvdb/io/io.h> // MappedFile
 #include <openvdb/io/Compression.h> // COMPRESS_BLOSC
 
@@ -144,12 +145,6 @@ public:
     /// Return a copy of this attribute.
     virtual AttributeArray::Ptr copy() const = 0;
 
-    /// Return a copy of this attribute.
-#ifndef _MSC_VER
-    OPENVDB_DEPRECATED_MESSAGE("In-memory compression no longer supported, use AttributeArray::copy() instead")
-#endif
-    virtual AttributeArray::Ptr copyUncompressed() const = 0;
-
     /// Return the number of elements in this array.
     /// @note This does not count each data element in a strided array
     virtual Index size() const = 0;
@@ -194,6 +189,12 @@ public:
     /// Return the number of bytes of memory used by this attribute.
     virtual size_t memUsage() const = 0;
 
+    /// Return the number of bytes of memory used by this attribute array once it
+    /// has been deserialized (this may be different to memUsage() if delay-loading
+    /// is in use). Note that this method does NOT consider the fact that a
+    /// uniform attribute could be expanded and only deals with delay-loading.
+    virtual size_t memUsageIfLoaded() const = 0;
+
     /// Create a new attribute array of the given (registered) type, length and stride.
     /// @details If @a lock is non-null, the AttributeArray registry mutex
     /// has already been locked
@@ -217,13 +218,6 @@ public:
     template<typename ValueType>
     bool hasValueType() const { return this->type().first == typeNameAsString<ValueType>(); }
 
-    /// @brief Set value at given index @a n from @a sourceIndex of another @a sourceArray.
-    // Windows does not allow base classes to be easily deprecated.
-#ifndef _MSC_VER
-    OPENVDB_DEPRECATED_MESSAGE("Use copyValues() with source-target index pairs")
-#endif
-    virtual void set(const Index n, const AttributeArray& sourceArray, const Index sourceIndex) = 0;
-
     /// @brief Copy values into this array from a source array to a target array
     /// as referenced by an iterator.
     /// @details Iterators must adhere to the ForwardIterator interface described
@@ -265,17 +259,6 @@ public:
     /// Compact the existing array to become uniform if all values are identical
     virtual bool compact() = 0;
 
-    // Windows does not allow base classes to be deprecated
-#ifndef _MSC_VER
-    OPENVDB_DEPRECATED_MESSAGE("Previously this compressed the attribute array, now it does nothing")
-#endif
-    virtual bool compress() = 0;
-    // Windows does not allow base classes to be deprecated
-#ifndef _MSC_VER
-    OPENVDB_DEPRECATED_MESSAGE("Previously this uncompressed the attribute array, now it does nothing")
-#endif
-    virtual bool decompress() = 0;
-
     /// @brief   Specify whether this attribute should be hidden (e.g., from UI or iterators).
     /// @details This is useful if the attribute is used for blind data or as scratch space
     ///          for a calculation.
@@ -344,10 +327,8 @@ public:
     bool operator==(const AttributeArray& other) const;
     bool operator!=(const AttributeArray& other) const { return !this->operator==(other); }
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     /// Indirect virtual function to retrieve the data buffer cast to a char byte array
     const char* constDataAsByteArray() const { return this->dataAsByteArray(); }
-#endif
 
 private:
     friend class ::TestAttributeArray;
@@ -366,9 +347,7 @@ private:
         bool rangeChecking = true);
 
 protected:
-#if OPENVDB_ABI_VERSION_NUMBER >= 7
     AttributeArray(const AttributeArray& rhs, const tbb::spin_mutex::scoped_lock&);
-#endif
 
     /// @brief Specify whether this attribute has a constant stride or not.
     void setConstantStride(bool state);
@@ -544,7 +523,7 @@ public:
     /// Default constructor, always constructs a uniform attribute.
     explicit TypedAttributeArray(Index n = 1, Index strideOrTotalSize = 1, bool constantStride = true,
         const ValueType& uniformValue = zeroVal<ValueType>());
-#if OPENVDB_ABI_VERSION_NUMBER >= 7
+
     /// Deep copy constructor.
     /// @note This method is thread-safe (as of ABI=7) for concurrently reading from the
     /// source attribute array while being deep-copied. Specifically, this means that the
@@ -552,15 +531,7 @@ public:
     /// while being copied using this copy-constructor in another thread.
     /// It is not thread-safe for write.
     TypedAttributeArray(const TypedAttributeArray&);
-    /// Deep copy constructor.
-    OPENVDB_DEPRECATED_MESSAGE("Use copy-constructor without unused bool parameter")
-    TypedAttributeArray(const TypedAttributeArray&, bool /*unused*/);
-#else
-    /// Deep copy constructor.
-    /// @note This method is not thread-safe for reading or writing, use
-    /// TypedAttributeArray::copy() to ensure thread-safety when reading concurrently.
-    TypedAttributeArray(const TypedAttributeArray&, bool uncompress = false);
-#endif
+
     /// Deep copy assignment operator.
     /// @note this operator is thread-safe.
     TypedAttributeArray& operator=(const TypedAttributeArray&);
@@ -575,11 +546,6 @@ public:
     /// @note This method is thread-safe.
     AttributeArray::Ptr copy() const override;
 
-    /// Return a copy of this attribute.
-    /// @note This method is thread-safe.
-    OPENVDB_DEPRECATED_MESSAGE("In-memory compression no longer supported, use AttributeArray::copy() instead")
-    AttributeArray::Ptr copyUncompressed() const override;
-
     /// Return a new attribute array of the given length @a n and @a stride with uniform value zero.
     static Ptr create(Index n, Index strideOrTotalSize = 1, bool constantStride = true,
         const Metadata* metadata = nullptr);
@@ -645,6 +611,12 @@ public:
     /// Return the number of bytes of memory used by this attribute.
     size_t memUsage() const override;
 
+    /// Return the number of bytes of memory used by this attribute array once it
+    /// has been deserialized (this may be different to memUsage() if delay-loading
+    /// is in use). Note that this method does NOT consider the fact that a
+    /// uniform attribute could be expanded and only deals with delay-loading.
+    size_t memUsageIfLoaded() const override;
+
     /// Return the value at index @a n (assumes in-core)
     ValueType getUnsafe(Index n) const;
     /// Return the value at index @a n
@@ -671,10 +643,6 @@ public:
     /// (assumes in-core)
     static void setUnsafe(AttributeArray* array, const Index n, const ValueType& value);
 
-    /// Set value at given index @a n from @a sourceIndex of another @a sourceArray
-    OPENVDB_DEPRECATED_MESSAGE("Use copyValues() with source-target index pairs")
-    void set(const Index n, const AttributeArray& sourceArray, const Index sourceIndex) override;
-
     /// Return @c true if this array is stored as a single uniform value.
     bool isUniform() const override { return mIsUniform; }
     /// @brief  Replace the single value storage with an array of length size().
@@ -697,13 +665,6 @@ public:
     /// Non-member equivalent to fill() that static_casts array to this TypedAttributeArray
     static void fill(AttributeArray* array, const ValueType& value);
 
-    /// Compress the attribute array.
-    OPENVDB_DEPRECATED_MESSAGE("Previously this compressed the attribute array, now it does nothing")
-    bool compress() override;
-    /// Uncompress the attribute array.
-    OPENVDB_DEPRECATED_MESSAGE("Previously this uncompressed the attribute array, now it does nothing")
-    bool decompress() override;
-
     /// Read attribute data from a stream.
     void read(std::istream&) override;
     /// Write attribute data to a stream.
@@ -744,17 +705,15 @@ public:
     /// Return @c true if all data has been loaded
     bool isDataLoaded() const override;
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     /// Return the raw data buffer
     inline const StorageType* constData() const { return this->data(); }
-#endif
 
 protected:
     AccessorBasePtr getAccessor() const override;
 
     /// Return the raw data buffer
-    inline StorageType* data() { assert(validData()); return mData.get(); }
-    inline const StorageType* data() const { assert(validData()); return mData.get(); }
+    inline StorageType* data() { OPENVDB_ASSERT(validData()); return mData.get(); }
+    inline const StorageType* data() const { OPENVDB_ASSERT(validData()); return mData.get(); }
 
     /// Verify that data is not out-of-core or in a partially-read state
     inline bool validData() const { return !(isOutOfCore() || (flags() & PARTIALREAD)); }
@@ -762,17 +721,12 @@ protected:
 private:
     friend class ::TestAttributeArray;
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 7
     TypedAttributeArray(const TypedAttributeArray&, const tbb::spin_mutex::scoped_lock&);
-#endif
 
     /// Load data from memory-mapped file.
     inline void doLoad() const;
     /// Load data from memory-mapped file (unsafe as this function is not protected by a mutex).
-    /// @param compression parameter no longer used
-    inline void doLoadUnsafe(const bool compression = true) const;
-    /// Compress in-core data assuming mutex is locked
-    inline bool compressUnsafe();
+    inline void doLoadUnsafe() const;
 
     /// Toggle out-of-core state
     inline void setOutOfCore(const bool);
@@ -794,7 +748,6 @@ private:
         return TypedAttributeArray::create(n, strideOrTotalSize, constantStride, metadata);
     }
 
-    static std::unique_ptr<const NamePair> sTypeName;
     std::unique_ptr<StorageType[]>      mData;
     Index                               mSize;
     Index                               mStrideOrTotalSize;
@@ -1011,17 +964,17 @@ void AttributeArray::doCopyValues(const AttributeArray& sourceArray, const IterT
     bool rangeChecking/*=true*/)
 {
     // ensure both arrays have float-float or integer-integer value types
-    assert(sourceArray.valueTypeIsFloatingPoint() == this->valueTypeIsFloatingPoint());
+    OPENVDB_ASSERT(sourceArray.valueTypeIsFloatingPoint() == this->valueTypeIsFloatingPoint());
     // ensure both arrays have been loaded from disk (if delay-loaded)
-    assert(sourceArray.isDataLoaded() && this->isDataLoaded());
+    OPENVDB_ASSERT(sourceArray.isDataLoaded() && this->isDataLoaded());
     // ensure storage size * stride matches on both arrays
-    assert(this->storageTypeSize()*this->stride() ==
+    OPENVDB_ASSERT(this->storageTypeSize()*this->stride() ==
         sourceArray.storageTypeSize()*sourceArray.stride());
 
     const size_t bytes(sourceArray.storageTypeSize()*sourceArray.stride());
     const char* const sourceBuffer = sourceArray.dataAsByteArray();
     char* const targetBuffer = this->dataAsByteArray();
-    assert(sourceBuffer && targetBuffer);
+    OPENVDB_ASSERT(sourceBuffer && targetBuffer);
 
     if (rangeChecking && this->isUniform()) {
         OPENVDB_THROW(IndexError, "Cannot copy array data as target array is uniform.");
@@ -1047,9 +1000,9 @@ void AttributeArray::doCopyValues(const AttributeArray& sourceArray, const IterT
             }
         } else {
             // range-checking asserts
-            assert(sourceIndex < sourceArray.dataSize());
-            assert(targetIndex < this->dataSize());
-            if (this->isUniform())  assert(targetIndex == Index(0));
+            OPENVDB_ASSERT(sourceIndex < sourceArray.dataSize());
+            OPENVDB_ASSERT(targetIndex < this->dataSize());
+            if (this->isUniform())  OPENVDB_ASSERT(targetIndex == Index(0));
         }
 
         const size_t targetOffset(targetIndex * bytes);
@@ -1100,9 +1053,6 @@ void AttributeArray::copyValues(const AttributeArray& sourceArray, const IterT&
 
 // TypedAttributeArray implementation
 
-template<typename ValueType_, typename Codec_>
-std::unique_ptr<const NamePair> TypedAttributeArray<ValueType_, Codec_>::sTypeName;
-
 
 template<typename ValueType_, typename Codec_>
 TypedAttributeArray<ValueType_, Codec_>::TypedAttributeArray(
@@ -1132,7 +1082,6 @@ TypedAttributeArray<ValueType_, Codec_>::TypedAttributeArray(
 }
 
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 7
 template<typename ValueType_, typename Codec_>
 TypedAttributeArray<ValueType_, Codec_>::TypedAttributeArray(const TypedAttributeArray& rhs)
     : TypedAttributeArray(rhs, tbb::spin_mutex::scoped_lock(rhs.mMutex))
@@ -1144,11 +1093,6 @@ template<typename ValueType_, typename Codec_>
 TypedAttributeArray<ValueType_, Codec_>::TypedAttributeArray(const TypedAttributeArray& rhs,
     const tbb::spin_mutex::scoped_lock& lock)
     : AttributeArray(rhs, lock)
-#else
-template<typename ValueType_, typename Codec_>
-TypedAttributeArray<ValueType_, Codec_>::TypedAttributeArray(const TypedAttributeArray& rhs, bool)
-    : AttributeArray(rhs)
-#endif
     , mSize(rhs.mSize)
     , mStrideOrTotalSize(rhs.mStrideOrTotalSize)
 {
@@ -1190,12 +1134,10 @@ template<typename ValueType_, typename Codec_>
 inline const NamePair&
 TypedAttributeArray<ValueType_, Codec_>::attributeType()
 {
-    static std::once_flag once;
-    std::call_once(once, []()
-    {
-        sTypeName.reset(new NamePair(typeNameAsString<ValueType>(), Codec::name()));
-    });
-    return *sTypeName;
+    static NamePair sTypeName = []() {
+        return NamePair(typeNameAsString<ValueType>(), Codec::name());
+    }();
+    return sTypeName;
 }
 
 
@@ -1259,21 +1201,10 @@ template<typename ValueType_, typename Codec_>
 AttributeArray::Ptr
 TypedAttributeArray<ValueType_, Codec_>::copy() const
 {
-#if OPENVDB_ABI_VERSION_NUMBER < 7
-    tbb::spin_mutex::scoped_lock lock(mMutex);
-#endif
     return AttributeArray::Ptr(new TypedAttributeArray<ValueType, Codec>(*this));
 }
 
 
-template<typename ValueType_, typename Codec_>
-AttributeArray::Ptr
-TypedAttributeArray<ValueType_, Codec_>::copyUncompressed() const
-{
-    return this->copy();
-}
-
-
 template<typename ValueType_, typename Codec_>
 size_t
 TypedAttributeArray<ValueType_, Codec_>::arrayMemUsage() const
@@ -1288,13 +1219,13 @@ template<typename ValueType_, typename Codec_>
 void
 TypedAttributeArray<ValueType_, Codec_>::allocate()
 {
-    assert(!mData);
+    OPENVDB_ASSERT(!mData);
     if (mIsUniform) {
         mData.reset(new StorageType[1]);
     }
     else {
         const size_t size(this->dataSize());
-        assert(size > 0);
+        OPENVDB_ASSERT(size > 0);
         mData.reset(new StorageType[size]);
     }
 }
@@ -1376,11 +1307,19 @@ TypedAttributeArray<ValueType_, Codec_>::memUsage() const
 }
 
 
+template<typename ValueType_, typename Codec_>
+size_t
+TypedAttributeArray<ValueType_, Codec_>::memUsageIfLoaded() const
+{
+    return sizeof(*this) + (mIsUniform ? 1 : this->dataSize()) * sizeof(StorageType);
+}
+
+
 template<typename ValueType_, typename Codec_>
 typename TypedAttributeArray<ValueType_, Codec_>::ValueType
 TypedAttributeArray<ValueType_, Codec_>::getUnsafe(Index n) const
 {
-    assert(n < this->dataSize());
+    OPENVDB_ASSERT(n < this->dataSize());
 
     ValueType val;
     Codec::decode(/*in=*/this->data()[mIsUniform ? 0 : n], /*out=*/val);
@@ -1429,9 +1368,9 @@ template<typename ValueType_, typename Codec_>
 void
 TypedAttributeArray<ValueType_, Codec_>::setUnsafe(Index n, const ValueType& val)
 {
-    assert(n < this->dataSize());
-    assert(!this->isOutOfCore());
-    assert(!this->isUniform());
+    OPENVDB_ASSERT(n < this->dataSize());
+    OPENVDB_ASSERT(!this->isOutOfCore());
+    OPENVDB_ASSERT(!this->isUniform());
 
     // this unsafe method assumes the data is not uniform, however if it is, this redirects the index
     // to zero, which is marginally less efficient but ensures not writing to an illegal address
@@ -1478,19 +1417,6 @@ TypedAttributeArray<ValueType_, Codec_>::setUnsafe(AttributeArray* array, const
 }
 
 
-template<typename ValueType_, typename Codec_>
-void
-TypedAttributeArray<ValueType_, Codec_>::set(Index n, const AttributeArray& sourceArray, const Index sourceIndex)
-{
-    const TypedAttributeArray& sourceTypedArray = static_cast<const TypedAttributeArray&>(sourceArray);
-
-    ValueType sourceValue;
-    sourceTypedArray.get(sourceIndex, sourceValue);
-
-    this->set(n, sourceValue);
-}
-
-
 template<typename ValueType_, typename Codec_>
 void
 TypedAttributeArray<ValueType_, Codec_>::expand(bool fill)
@@ -1584,30 +1510,6 @@ TypedAttributeArray<ValueType_, Codec_>::fill(AttributeArray* array, const Value
 }
 
 
-template<typename ValueType_, typename Codec_>
-inline bool
-TypedAttributeArray<ValueType_, Codec_>::compress()
-{
-    return false;
-}
-
-
-template<typename ValueType_, typename Codec_>
-inline bool
-TypedAttributeArray<ValueType_, Codec_>::compressUnsafe()
-{
-    return false;
-}
-
-
-template<typename ValueType_, typename Codec_>
-inline bool
-TypedAttributeArray<ValueType_, Codec_>::decompress()
-{
-    return false;
-}
-
-
 template<typename ValueType_, typename Codec_>
 bool
 TypedAttributeArray<ValueType_, Codec_>::isOutOfCore() const
@@ -1732,7 +1634,7 @@ TypedAttributeArray<ValueType_, Codec_>::readBuffers(std::istream& is)
     uint8_t bloscCompressed(0);
     if (!mIsUniform)    is.read(reinterpret_cast<char*>(&bloscCompressed), sizeof(uint8_t));
 
-    assert(mFlags & PARTIALREAD);
+    OPENVDB_ASSERT(mFlags & PARTIALREAD);
     std::unique_ptr<char[]> buffer(new char[mCompressedBytes]);
     is.read(buffer.get(), mCompressedBytes);
     mCompressedBytes = 0;
@@ -1764,35 +1666,45 @@ TypedAttributeArray<ValueType_, Codec_>::readPagedBuffers(compression::PagedInpu
         return;
     }
 
+#ifdef OPENVDB_USE_DELAYED_LOADING
     // If this array is being read from a memory-mapped file, delay loading of its data
     // until the data is actually accessed.
     io::MappedFile::Ptr mappedFile = io::getMappedFilePtr(is.getInputStream());
     const bool delayLoad = (mappedFile.get() != nullptr);
+#endif
 
     if (is.sizeOnly())
     {
         size_t compressedBytes(mCompressedBytes);
         mCompressedBytes = 0; // if not set to zero, mPageHandle will attempt to destroy invalid memory
         mFlags = static_cast<uint8_t>(mFlags & ~PARTIALREAD); // mark data read as having completed
-        assert(!mPageHandle);
+        OPENVDB_ASSERT(!mPageHandle);
         mPageHandle = is.createHandle(compressedBytes);
         return;
     }
 
-    assert(mPageHandle);
+    OPENVDB_ASSERT(mPageHandle);
 
     tbb::spin_mutex::scoped_lock lock(mMutex);
 
     this->deallocate();
 
+#ifdef OPENVDB_USE_DELAYED_LOADING
     this->setOutOfCore(delayLoad);
     is.read(mPageHandle, std::streamsize(mPageHandle->size()), delayLoad);
+#else
+    is.read(mPageHandle, std::streamsize(mPageHandle->size()), false);
+#endif // OPENVDB_USE_DELAYED_LOADING
 
+#ifdef OPENVDB_USE_DELAYED_LOADING
     if (!delayLoad) {
+#endif
         std::unique_ptr<char[]> buffer = mPageHandle->read();
         mData.reset(reinterpret_cast<StorageType*>(buffer.release()));
         mPageHandle.reset();
+#ifdef OPENVDB_USE_DELAYED_LOADING
     }
+#endif
 
     // clear page state
 
@@ -1830,7 +1742,7 @@ TypedAttributeArray<ValueType_, Codec_>::writeMetadata(std::ostream& os, bool ou
     uint8_t flags(mFlags);
     uint8_t serializationFlags(0);
     Index size(mSize);
-    Index stride(mStrideOrTotalSize);
+    Index strideOrTotalSize(mStrideOrTotalSize);
     bool strideOfOne(this->stride() == 1);
 
     bool bloscCompression = io::getDataCompression(os) & io::COMPRESS_BLOSC;
@@ -1872,7 +1784,7 @@ TypedAttributeArray<ValueType_, Codec_>::writeMetadata(std::ostream& os, bool ou
     os.write(reinterpret_cast<const char*>(&size), sizeof(Index));
 
     // write strided
-    if (!strideOfOne)       os.write(reinterpret_cast<const char*>(&stride), sizeof(Index));
+    if (!strideOfOne) os.write(reinterpret_cast<const char*>(&strideOrTotalSize), sizeof(Index));
 }
 
 
@@ -1943,7 +1855,7 @@ TypedAttributeArray<ValueType_, Codec_>::writePagedBuffers(compression::PagedOut
 
 template<typename ValueType_, typename Codec_>
 void
-TypedAttributeArray<ValueType_, Codec_>::doLoadUnsafe(const bool /*compression*/) const
+TypedAttributeArray<ValueType_, Codec_>::doLoadUnsafe() const
 {
     if (!(this->isOutOfCore())) return;
 
@@ -1951,8 +1863,8 @@ TypedAttributeArray<ValueType_, Codec_>::doLoadUnsafe(const bool /*compression*/
 
     auto* self = const_cast<TypedAttributeArray<ValueType_, Codec_>*>(this);
 
-    assert(self->mPageHandle);
-    assert(!(self->mFlags & PARTIALREAD));
+    OPENVDB_ASSERT(self->mPageHandle);
+    OPENVDB_ASSERT(!(self->mFlags & PARTIALREAD));
 
     std::unique_ptr<char[]> buffer = self->mPageHandle->read();
 
@@ -2093,7 +2005,7 @@ AttributeHandle<ValueType, CodecType>::AttributeHandle(const AttributeArray& arr
     // bind getter and setter methods
 
     AttributeArray::AccessorBasePtr accessor = mArray->getAccessor();
-    assert(accessor);
+    OPENVDB_ASSERT(accessor);
 
     AttributeArray::Accessor<ValueType>* typedAccessor = static_cast<AttributeArray::Accessor<ValueType>*>(accessor.get());
 
@@ -2133,7 +2045,7 @@ AttributeHandle<ValueType, CodecType>::compatibleType() const
 template <typename ValueType, typename CodecType>
 const AttributeArray& AttributeHandle<ValueType, CodecType>::array() const
 {
-    assert(mArray);
+    OPENVDB_ASSERT(mArray);
     return *mArray;
 }
 
@@ -2141,7 +2053,7 @@ template <typename ValueType, typename CodecType>
 Index AttributeHandle<ValueType, CodecType>::index(Index n, Index m) const
 {
     Index index = n * mStrideOrTotalSize + m;
-    assert(index < (mSize * mStrideOrTotalSize));
+    OPENVDB_ASSERT(index < (mSize * mStrideOrTotalSize));
     return index;
 }
 
@@ -2267,7 +2179,7 @@ AttributeWriteHandle<ValueType, CodecType>::set(Index index, const ValueType& va
 template <typename ValueType, typename CodecType>
 AttributeArray& AttributeWriteHandle<ValueType, CodecType>::array()
 {
-    assert(this->mArray);
+    OPENVDB_ASSERT(this->mArray);
     return *const_cast<AttributeArray*>(this->mArray);
 }
 
