@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #ifndef OPENVDB_TOOLS_DENSESPARSETOOLS_HAS_BEEN_INCLUDED
 #define OPENVDB_TOOLS_DENSESPARSETOOLS_HAS_BEEN_INCLUDED
@@ -371,7 +371,7 @@ public:
 
         // Clean up an unused leaf.
 
-        if (leaf != nullptr) delete leaf;
+        delete leaf;
     }
 
     void join(SparseExtractor& rhs) {
@@ -530,7 +530,7 @@ public:
 
         // Clean up an unused leaf.
 
-        if (leaf != nullptr) delete leaf;
+        delete leaf;
     }
 
     void join(SparseMaskedExtractor& rhs) {
