@@ -1,11 +1,12 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #ifndef OPENVDB_TREE_LEAF_NODE_BOOL_HAS_BEEN_INCLUDED
 #define OPENVDB_TREE_LEAF_NODE_BOOL_HAS_BEEN_INCLUDED
 
 #include <openvdb/Types.h>
 #include <openvdb/io/Compression.h> // for io::readData(), etc.
+#include <openvdb/util/Assert.h>
 #include <openvdb/math/Math.h> // for math::isZero()
 #include <openvdb/util/NodeMasks.h>
 #include "LeafNode.h"
@@ -79,13 +80,35 @@ public:
     template<typename OtherValueType>
     explicit LeafNode(const LeafNode<OtherValueType, Log2Dim>& other);
 
-    /// Topology copy constructor
+    /// @brief Deprecated topology copy constructor
+    /// @note  This constructor initialises the bool buffer to the ValueMask
+    ///   states (i.e. value will be true if the active state is on and
+    ///   vice-versa). This is not really a "TopologyCopy" and is therefor
+    ///   deprecated. Use the explicit mask/buffer constructor instead:
+    /// @code
+    ///    // build new leaf node with the mask of 'a', but with the mask of
+    ///    // 'b' as the the new value buffer.
+    ///    const LeafNode a = ... ;
+    ///    const LeafNode b = ... ;
+    ///    const LeafNode copy(a.origin(), /*mask=*/a.getValueMask(),
+    ///      /*buff=*/b.getValueMask());
+    /// @endcode
     template<typename ValueType>
+    OPENVDB_DEPRECATED_MESSAGE("Use LeafNodeBool component constructor.")
     LeafNode(const LeafNode<ValueType, Log2Dim>& other, TopologyCopy);
 
+    /// @brief Construct a LeafNodeBool with its individual components
+    /// @param xyz  Leaf origin
+    /// @param mask  The ValueMask to copy
+    /// @param buff  The LeafBuffer to copy (also constructable from a ValueMask)
+    /// @param trans Transient data (ignored until ABI 9)
+    LeafNode(const Coord& xyz,
+        const NodeMaskType& mask,
+        const Buffer& buff,
+        const Index32 trans = 0);
+
     //@{
-    /// @brief Topology copy constructor
-    /// @note This variant exists mainly to enable template instantiation.
+    /// @brief Topology copy constructors
     template<typename ValueType>
     LeafNode(const LeafNode<ValueType, Log2Dim>& other, bool offValue, bool onValue, TopologyCopy);
     template<typename ValueType>
@@ -108,10 +131,12 @@ public:
     static void getNodeLog2Dims(std::vector<Index>& dims) { dims.push_back(Log2Dim); }
     static Index getChildDim() { return 1; }
 
-    static Index32 leafCount() { return 1; }
+    static Index64 leafCount() { return 1; }
     /// no-op
+    void nodeCount(std::vector<Index64> &) const {}
+    OPENVDB_DEPRECATED_MESSAGE("Use input type std::vector<Index64> for nodeCount.")
     void nodeCount(std::vector<Index32> &) const {}
-    static Index32 nonLeafCount() { return 0; }
+    static Index64 nonLeafCount() { return 0; }
 
     /// Return the number of active voxels.
     Index64 onVoxelCount() const { return mValueMask.countOn(); }
@@ -137,6 +162,7 @@ public:
 
     /// Return the memory in bytes occupied by this node.
     Index64 memUsage() const;
+    Index64 memUsageIfLoaded() const;
 
     /// Expand the given bounding box so that it includes this leaf node's active voxels.
     /// If visitVoxels is false this LeafNode will be approximated as dense, i.e. with all
@@ -164,12 +190,10 @@ public:
     /// Return the global coordinates for a linear table offset.
     Coord offsetToGlobalCoord(Index n) const;
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     /// Return the transient data value.
     Index32 transientData() const { return mTransientData; }
     /// Set the transient data value.
     void setTransientData(Index32 transientData) { mTransientData = transientData; }
-#endif
 
     /// Return a string representation of this node.
     std::string str() const;
@@ -218,6 +242,10 @@ public:
     /// @param xyz       the coordinates of the voxel to be probed
     /// @param[out] val  the value of the voxel at the given coordinates
     bool probeValue(const Coord& xyz, bool& val) const;
+    /// @brief Return @c true if the voxel at the given offset is active.
+    /// @param offset    the linear offset of the voxel to be probed
+    /// @param[out] val  the value of the voxel at the given coordinates
+    bool probeValue(Index offset, bool& val) const;
 
     /// Return the level (0) at which leaf node values reside.
     static Index getValueLevel(const Coord&) { return LEVEL; }
@@ -225,17 +253,17 @@ public:
     /// Set the active state of the voxel at the given coordinates but don't change its value.
     void setActiveState(const Coord& xyz, bool on);
     /// Set the active state of the voxel at the given offset but don't change its value.
-    void setActiveState(Index offset, bool on) { assert(offset<SIZE); mValueMask.set(offset, on); }
+    void setActiveState(Index offset, bool on) { OPENVDB_ASSERT(offset<SIZE); mValueMask.set(offset, on); }
 
     /// Set the value of the voxel at the given coordinates but don't change its active state.
     void setValueOnly(const Coord& xyz, bool val);
     /// Set the value of the voxel at the given offset but don't change its active state.
-    void setValueOnly(Index offset, bool val) { assert(offset<SIZE); mBuffer.setValue(offset,val); }
+    void setValueOnly(Index offset, bool val) { OPENVDB_ASSERT(offset<SIZE); mBuffer.setValue(offset,val); }
 
     /// Mark the voxel at the given coordinates as inactive but don't change its value.
     void setValueOff(const Coord& xyz) { mValueMask.setOff(this->coordToOffset(xyz)); }
     /// Mark the voxel at the given offset as inactive but don't change its value.
-    void setValueOff(Index offset) { assert(offset < SIZE); mValueMask.setOff(offset); }
+    void setValueOff(Index offset) { OPENVDB_ASSERT(offset < SIZE); mValueMask.setOff(offset); }
 
     /// Set the value of the voxel at the given coordinates and mark the voxel as inactive.
     void setValueOff(const Coord& xyz, bool val);
@@ -245,7 +273,7 @@ public:
     /// Mark the voxel at the given coordinates as active but don't change its value.
     void setValueOn(const Coord& xyz) { mValueMask.setOn(this->coordToOffset(xyz)); }
     /// Mark the voxel at the given offset as active but don't change its value.
-    void setValueOn(Index offset) { assert(offset < SIZE); mValueMask.setOn(offset); }
+    void setValueOn(Index offset) { OPENVDB_ASSERT(offset < SIZE); mValueMask.setOn(offset); }
 
     /// Set the value of the voxel at the given coordinates and mark the voxel as active.
     void setValueOn(const Coord& xyz, bool val);
@@ -273,9 +301,13 @@ public:
     void setValuesOff() { mValueMask.setOff(); }
 
     /// Return @c true if the voxel at the given coordinates is active.
-    bool isValueOn(const Coord& xyz) const { return mValueMask.isOn(this->coordToOffset(xyz)); }
+    bool isValueOn(const Coord& xyz) const { return this->isValueOn(this->coordToOffset(xyz)); }
     /// Return @c true if the voxel at the given offset is active.
-    bool isValueOn(Index offset) const { assert(offset < SIZE); return mValueMask.isOn(offset); }
+    bool isValueOn(Index offset) const { OPENVDB_ASSERT(offset < SIZE); return mValueMask.isOn(offset); }
+    /// Return @c true if the voxel at the given coordinates is inactive.
+    bool isValueOff(const Coord& xyz) const { return this->isValueOff(this->coordToOffset(xyz)); }
+    /// Return @c true if the voxel at the given offset is inactive.
+    bool isValueOff(Index offset) const { OPENVDB_ASSERT(offset < SIZE); return mValueMask.isOff(offset); }
 
     /// Return @c false since leaf nodes never contain tiles.
     static bool hasActiveTiles() { return false; }
@@ -435,6 +467,29 @@ public:
     /// Return @c true if all of this node's values are inactive.
     bool isInactive() const { return mValueMask.isOff(); }
 
+    //
+    // Unsafe methods
+    //
+    // These methods are not in fact unsafe, but are only offered so that
+    // the same methods can be called on both internal nodes and leaf nodes.
+
+    /// Return the value of the voxel at the given offset.
+    const bool& getValueUnsafe(Index offset) const { return this->getValue(offset); }
+    /// Return true if the voxel at the given offset is active and set value.
+    bool getValueUnsafe(Index offset, bool& value) const { return this->probeValue(offset, value); }
+    /// Set the active state of the voxel at the given offset but don't change its value.
+    void setActiveStateUnsafe(Index offset, bool on) { this->setActiveState(offset, on); }
+    /// Set the value of the voxel at the given coordinates but don't change its active state.
+    void setValueOnlyUnsafe(Index offset, const bool& value) { return this->setValueOnly(offset, value); }
+    /// Mark the voxel at the given offset as active but don't change its value.
+    void setValueOnUnsafe(Index offset) { this->setValueOn(offset); }
+    /// Set the value of the voxel at the given coordinates and mark the voxel as active.
+    void setValueOnUnsafe(Index offset, const bool& value) { this->setValueOn(offset, value); }
+    /// Mark the voxel at the given offset as inactive but don't change its value.
+    void setValueOffUnsafe(Index offset) { this->setValueOff(offset); }
+    /// Set the value of the voxel at the given coordinates and mark the voxel as active.
+    void setValueOffUnsafe(Index offset, const bool& value) { this->setValueOff(offset, value); }
+
     void resetBackground(bool oldBackground, bool newBackground);
 
     void negate() { mBuffer.mData.toggle(); }
@@ -496,24 +551,6 @@ public:
     template<typename CombineOp, typename OtherNodeT /*= LeafNode*/>
     void combine2(const LeafNode& b0, const OtherNodeT& b1, CombineOp&);
 
-    /// @brief Calls the templated functor BBoxOp with bounding box information.
-    /// An additional level argument is provided to the callback.
-    ///
-    /// @note The bounding boxes are guaranteed to be non-overlapping.
-    template<typename BBoxOp> void visitActiveBBox(BBoxOp&) const;
-
-    template<typename VisitorOp> void visit(VisitorOp&);
-    template<typename VisitorOp> void visit(VisitorOp&) const;
-
-    template<typename OtherLeafNodeType, typename VisitorOp>
-    void visit2Node(OtherLeafNodeType& other, VisitorOp&);
-    template<typename OtherLeafNodeType, typename VisitorOp>
-    void visit2Node(OtherLeafNodeType& other, VisitorOp&) const;
-    template<typename IterT, typename VisitorOp>
-    void visit2(IterT& otherIter, VisitorOp&, bool otherIsLHS = false);
-    template<typename IterT, typename VisitorOp>
-    void visit2(IterT& otherIter, VisitorOp&, bool otherIsLHS = false) const;
-
     //@{
     /// This function exists only to enable template instantiation.
     void prune(const ValueType& /*tolerance*/ = zeroVal<ValueType>()) {}
@@ -717,28 +754,14 @@ protected:
     /// Compute the origin of the leaf node that contains the voxel with the given coordinates.
     static void evalNodeOrigin(Coord& xyz) { xyz &= ~(DIM - 1); }
 
-    template<typename NodeT, typename VisitorOp, typename ChildAllIterT>
-    static inline void doVisit(NodeT&, VisitorOp&);
-
-    template<typename NodeT, typename OtherNodeT, typename VisitorOp,
-        typename ChildAllIterT, typename OtherChildAllIterT>
-    static inline void doVisit2Node(NodeT& self, OtherNodeT& other, VisitorOp&);
-
-    template<typename NodeT, typename VisitorOp,
-        typename ChildAllIterT, typename OtherChildAllIterT>
-    static inline void doVisit2(NodeT& self, OtherChildAllIterT&, VisitorOp&, bool otherIsLHS);
-
-
     /// Bitmask that determines which voxels are active
     NodeMaskType mValueMask;
     /// Bitmask representing the values of voxels
     Buffer mBuffer;
     /// Global grid index coordinates (x,y,z) of the local origin of this node
     Coord mOrigin;
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     /// Transient data (not serialized)
     Index32 mTransientData = 0;
-#endif
 
 private:
     /// @brief During topology-only construction, access is needed
@@ -803,9 +826,7 @@ LeafNode<bool, Log2Dim>::LeafNode(const LeafNode &other)
     : mValueMask(other.valueMask())
     , mBuffer(other.mBuffer)
     , mOrigin(other.mOrigin)
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     , mTransientData(other.mTransientData)
-#endif
 {
 }
 
@@ -817,9 +838,7 @@ inline
 LeafNode<bool, Log2Dim>::LeafNode(const LeafNode<ValueT, Log2Dim>& other)
     : mValueMask(other.valueMask())
     , mOrigin(other.origin())
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     , mTransientData(other.mTransientData)
-#endif
 {
     struct Local {
         /// @todo Consider using a value conversion functor passed as an argument instead.
@@ -840,9 +859,7 @@ LeafNode<bool, Log2Dim>::LeafNode(const LeafNode<ValueT, Log2Dim>& other,
     : mValueMask(other.valueMask())
     , mBuffer(background)
     , mOrigin(other.origin())
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     , mTransientData(other.mTransientData)
-#endif
 {
 }
 
@@ -854,12 +871,22 @@ LeafNode<bool, Log2Dim>::LeafNode(const LeafNode<ValueT, Log2Dim>& other, Topolo
     : mValueMask(other.valueMask())
     , mBuffer(other.valueMask())// value = active state
     , mOrigin(other.origin())
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     , mTransientData(other.mTransientData)
-#endif
 {
 }
 
+template<Index Log2Dim>
+inline
+LeafNode<bool, Log2Dim>::LeafNode(const Coord& xyz,
+    const NodeMaskType& mask,
+    const Buffer& buff,
+    const Index32 trans)
+    : mValueMask(mask)
+    , mBuffer(buff)
+    , mOrigin(xyz & (~(DIM - 1)))
+    , mTransientData(trans)
+{
+}
 
 template<Index Log2Dim>
 template<typename ValueT>
@@ -867,13 +894,15 @@ inline
 LeafNode<bool, Log2Dim>::LeafNode(const LeafNode<ValueT, Log2Dim>& other,
                                   bool offValue, bool onValue, TopologyCopy)
     : mValueMask(other.valueMask())
-    , mBuffer(other.valueMask())
+    , mBuffer(offValue)
     , mOrigin(other.origin())
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     , mTransientData(other.mTransientData)
-#endif
 {
-    if (offValue) { if (!onValue) mBuffer.mData.toggle(); else mBuffer.mData.setOn(); }
+    for (Index i = 0; i < SIZE; ++i) {
+        if (mValueMask.isOn(i)) {
+            mBuffer.setValue(i, onValue);
+        }
+    }
 }
 
 
@@ -896,6 +925,15 @@ LeafNode<bool, Log2Dim>::memUsage() const
 }
 
 
+template<Index Log2Dim>
+inline Index64
+LeafNode<bool, Log2Dim>::memUsageIfLoaded() const
+{
+    // Use sizeof(*this) to capture alignment-related padding
+    return sizeof(*this);
+}
+
+
 template<Index Log2Dim>
 inline void
 LeafNode<bool, Log2Dim>::evalActiveBoundingBox(CoordBBox& bbox, bool visitVoxels) const
@@ -918,7 +956,7 @@ template<typename OtherType, Index OtherLog2Dim>
 inline bool
 LeafNode<bool, Log2Dim>::hasSameTopology(const LeafNode<OtherType, OtherLog2Dim>* other) const
 {
-    assert(other);
+    OPENVDB_ASSERT(other);
     return (Log2Dim == OtherLog2Dim && mValueMask == other->getValueMask());
 }
 
@@ -941,7 +979,7 @@ template<Index Log2Dim>
 inline Index
 LeafNode<bool, Log2Dim>::coordToOffset(const Coord& xyz)
 {
-    assert ((xyz[0] & (DIM-1u)) < DIM && (xyz[1] & (DIM-1u)) < DIM && (xyz[2] & (DIM-1u)) < DIM);
+    OPENVDB_ASSERT((xyz[0] & (DIM-1u)) < DIM && (xyz[1] & (DIM-1u)) < DIM && (xyz[2] & (DIM-1u)) < DIM);
     return ((xyz[0] & (DIM-1u)) << 2*Log2Dim)
          + ((xyz[1] & (DIM-1u)) << Log2Dim)
          +  (xyz[2] & (DIM-1u));
@@ -952,7 +990,7 @@ template<Index Log2Dim>
 inline Coord
 LeafNode<bool, Log2Dim>::offsetToLocalCoord(Index n)
 {
-    assert(n < (1 << 3*Log2Dim));
+    OPENVDB_ASSERT(n < (1 << 3*Log2Dim));
     Coord xyz;
     xyz.setX(n >> 2*Log2Dim);
     n &= ((1 << 2*Log2Dim) - 1);
@@ -1142,7 +1180,7 @@ template<Index Log2Dim>
 inline void
 LeafNode<bool, Log2Dim>::addTile(Index offset, bool val, bool active)
 {
-    assert(offset < SIZE);
+    OPENVDB_ASSERT(offset < SIZE);
     this->setValueOnly(offset, val);
     this->setActiveState(offset, active);
 }
@@ -1173,7 +1211,7 @@ template<Index Log2Dim>
 inline const bool&
 LeafNode<bool, Log2Dim>::getValue(Index offset) const
 {
-    assert(offset < SIZE);
+    OPENVDB_ASSERT(offset < SIZE);
     // This *CANNOT* use operator ? for Windows
     if (mBuffer.mData.isOn(offset)) return Buffer::sOn; else return Buffer::sOff;
 }
@@ -1183,12 +1221,18 @@ template<Index Log2Dim>
 inline bool
 LeafNode<bool, Log2Dim>::probeValue(const Coord& xyz, bool& val) const
 {
-    const Index offset = this->coordToOffset(xyz);
+    return this->probeValue(this->coordToOffset(xyz), val);
+}
+
+template<Index Log2Dim>
+inline bool
+LeafNode<bool, Log2Dim>::probeValue(Index offset, bool& val) const
+{
+    OPENVDB_ASSERT(offset < SIZE);
     val = mBuffer.mData.isOn(offset);
     return mValueMask.isOn(offset);
 }
 
-
 template<Index Log2Dim>
 inline void
 LeafNode<bool, Log2Dim>::setValueOn(const Coord& xyz, bool val)
@@ -1201,7 +1245,7 @@ template<Index Log2Dim>
 inline void
 LeafNode<bool, Log2Dim>::setValueOn(Index offset, bool val)
 {
-    assert(offset < SIZE);
+    OPENVDB_ASSERT(offset < SIZE);
     mValueMask.setOn(offset);
     mBuffer.mData.set(offset, val);
 }
@@ -1235,7 +1279,7 @@ template<Index Log2Dim>
 inline void
 LeafNode<bool, Log2Dim>::setValueOff(Index offset, bool val)
 {
-    assert(offset < SIZE);
+    OPENVDB_ASSERT(offset < SIZE);
     mValueMask.setOff(offset);
     mBuffer.mData.set(offset, val);
 }
@@ -1606,144 +1650,6 @@ LeafNode<bool, Log2Dim>::combine2(const LeafNode& b0, const OtherNodeT& b1, Comb
 }
 
 
-////////////////////////////////////////
-
-template<Index Log2Dim>
-template<typename BBoxOp>
-inline void
-LeafNode<bool, Log2Dim>::visitActiveBBox(BBoxOp& op) const
-{
-    if (op.template descent<LEVEL>()) {
-        for (ValueOnCIter i=this->cbeginValueOn(); i; ++i) {
-            op.template operator()<LEVEL>(CoordBBox::createCube(i.getCoord(), 1));
-        }
-    } else {
-        op.template operator()<LEVEL>(this->getNodeBoundingBox());
-    }
-}
-
-
-template<Index Log2Dim>
-template<typename VisitorOp>
-inline void
-LeafNode<bool, Log2Dim>::visit(VisitorOp& op)
-{
-    doVisit<LeafNode, VisitorOp, ChildAllIter>(*this, op);
-}
-
-
-template<Index Log2Dim>
-template<typename VisitorOp>
-inline void
-LeafNode<bool, Log2Dim>::visit(VisitorOp& op) const
-{
-    doVisit<const LeafNode, VisitorOp, ChildAllCIter>(*this, op);
-}
-
-
-template<Index Log2Dim>
-template<typename NodeT, typename VisitorOp, typename ChildAllIterT>
-inline void
-LeafNode<bool, Log2Dim>::doVisit(NodeT& self, VisitorOp& op)
-{
-    for (ChildAllIterT iter = self.beginChildAll(); iter; ++iter) {
-        op(iter);
-    }
-}
-
-
-////////////////////////////////////////
-
-
-template<Index Log2Dim>
-template<typename OtherLeafNodeType, typename VisitorOp>
-inline void
-LeafNode<bool, Log2Dim>::visit2Node(OtherLeafNodeType& other, VisitorOp& op)
-{
-    doVisit2Node<LeafNode, OtherLeafNodeType, VisitorOp, ChildAllIter,
-        typename OtherLeafNodeType::ChildAllIter>(*this, other, op);
-}
-
-
-template<Index Log2Dim>
-template<typename OtherLeafNodeType, typename VisitorOp>
-inline void
-LeafNode<bool, Log2Dim>::visit2Node(OtherLeafNodeType& other, VisitorOp& op) const
-{
-    doVisit2Node<const LeafNode, OtherLeafNodeType, VisitorOp, ChildAllCIter,
-        typename OtherLeafNodeType::ChildAllCIter>(*this, other, op);
-}
-
-
-template<Index Log2Dim>
-template<
-    typename NodeT,
-    typename OtherNodeT,
-    typename VisitorOp,
-    typename ChildAllIterT,
-    typename OtherChildAllIterT>
-inline void
-LeafNode<bool, Log2Dim>::doVisit2Node(NodeT& self, OtherNodeT& other, VisitorOp& op)
-{
-    // Allow the two nodes to have different ValueTypes, but not different dimensions.
-    static_assert(OtherNodeT::SIZE == NodeT::SIZE,
-        "can't visit nodes of different sizes simultaneously");
-    static_assert(OtherNodeT::LEVEL == NodeT::LEVEL,
-        "can't visit nodes at different tree levels simultaneously");
-
-    ChildAllIterT iter = self.beginChildAll();
-    OtherChildAllIterT otherIter = other.beginChildAll();
-
-    for ( ; iter && otherIter; ++iter, ++otherIter) {
-        op(iter, otherIter);
-    }
-}
-
-
-////////////////////////////////////////
-
-
-template<Index Log2Dim>
-template<typename IterT, typename VisitorOp>
-inline void
-LeafNode<bool, Log2Dim>::visit2(IterT& otherIter, VisitorOp& op, bool otherIsLHS)
-{
-    doVisit2<LeafNode, VisitorOp, ChildAllIter, IterT>(*this, otherIter, op, otherIsLHS);
-}
-
-
-template<Index Log2Dim>
-template<typename IterT, typename VisitorOp>
-inline void
-LeafNode<bool, Log2Dim>::visit2(IterT& otherIter, VisitorOp& op, bool otherIsLHS) const
-{
-    doVisit2<const LeafNode, VisitorOp, ChildAllCIter, IterT>(*this, otherIter, op, otherIsLHS);
-}
-
-
-template<Index Log2Dim>
-template<
-    typename NodeT,
-    typename VisitorOp,
-    typename ChildAllIterT,
-    typename OtherChildAllIterT>
-inline void
-LeafNode<bool, Log2Dim>::doVisit2(NodeT& self, OtherChildAllIterT& otherIter,
-    VisitorOp& op, bool otherIsLHS)
-{
-    if (!otherIter) return;
-
-    if (otherIsLHS) {
-        for (ChildAllIterT iter = self.beginChildAll(); iter; ++iter) {
-            op(otherIter, iter);
-        }
-    } else {
-        for (ChildAllIterT iter = self.beginChildAll(); iter; ++iter) {
-            op(iter, otherIter);
-        }
-    }
-}
-
 } // namespace tree
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
